import sys
import traceback

from cpl_core.application import ApplicationABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_query.extension import List

from py_to_uxf_core.abc.python_parser_abc import PythonParserABC
from py_to_uxf_core.abc.umlet_creator_abc import UmletCreatorABC
from py_to_uxf_core.model.class_implementation import ClassImplementation
from py_to_uxf_core.model.python_class import PythonClass


class Application(ApplicationABC):

    def __init__(self, config: ConfigurationABC, services: ServiceProviderABC):
        ApplicationABC.__init__(self, config, services)

        self._path = config.get_configuration('p')
        self._file = ''
        try:
            self._file = config.get_configuration('-pAdditionalArguments')[1]
        except Exception as e:
            Console.error('Expected file')
            sys.exit()
        self._parser: PythonParserABC = services.get_service(PythonParserABC)
        self._umlet_creator: UmletCreatorABC = services.get_service(UmletCreatorABC)

    def configure(self):
        pass

    @staticmethod
    def _console_output(classes: list[PythonClass]):
        for cls in classes:
            Console.write_line(f'Class {cls.name}')
            if len(cls.attributes) > 0:
                Console.write_line('\tAttributes:')
            for atr in cls.attributes:
                Console.write_line(f'\t{atr.access_modifier.value}{atr.name}: {atr.type}')

            if len(cls.functions) > 0:
                Console.write_line('\tFunctions:')
            for func in cls.functions:
                args = ''
                Console.write_line(f'\t{func.access_modifier.value}{func.name}({args}): {func.return_type}')
            Console.write_line()

    @staticmethod
    def _exit(msg: str, tb=None):
        if tb is not None:
            Console.error(msg, tb)
        else:
            Console.error(msg)
        sys.exit()

    def main(self):
        Console.banner('Py to UXF')
        if self._path is None:
            self._exit('Expected path\n')

        if self._file is None:
            self._exit(f'Expected output file\n')

        Console.write_line(f'Input path:', self._path)
        Console.write_line(f'Output path:', self._file)
        xml = ''
        try:
            classes: List[PythonClass] = self._parser.parse_classes()
            Console.write_line(f'Found {classes.count()} classes')
            implementations: List[ClassImplementation] = self._parser.parse_implementations(classes)
            Console.write_line(f'Found {implementations.count()} implementations')
            # self._console_output(classes)
            xml = self._umlet_creator.generate_xml(classes, implementations)
        except Exception as e:
            self._exit('Parsing failed', f'{e} -> {traceback.format_exc()}\n')

        if not self._file.endswith('.uxf'):
            self._exit(f'Unexpected file {self._file}\n')

        with open(self._file, 'w+') as file:
            file.write(xml)
            file.close()

        Console.write_line('\n')
